local eventFrame = CreateFrame("Frame");
local QuickHealth = LibStub("LibQuickHealth-1.0");
local _G = getfenv(0);

-- As we have no easy way to itterate through all blizz frames we'll make a table to itterate through
local FrameMap =
{
	["player"] = "PlayerFrame",
	["target"] = "TargetFrame",
	["focus"] = "FocusFrame",
};

for i = 1, 5 do
	FrameMap[("party%d"):format(i)] = ("PartyMemberFrame%d"):format(i);
end
for i = 1, 40 do
	FrameMap[("raid%d"):format(i)] = ("RaidGroupButton%d"):format(i);
end

-- Map that will contain the units of every guid
local GUIDMap = {};

-- This function will regenerate the map
local function RemapGUIDMap()
	for key in pairs(GUIDMap) do
		GUIDMap[key] = nil;
	end
	for unit in pairs(FrameMap) do
		local guid = UnitGUID(unit);
		if(guid) then
			if(not GUIDMap[guid]) then
				GUIDMap[guid] = {};
			end
			table.insert(GUIDMap[guid], unit)
		end
	end
end


local handlerTable = {};
eventFrame:SetScript("OnEvent", function(self, event, ...)
	handlerTable[event](handlerTable, ...);
end);


function handlerTable.PLAYER_LOGIN(self)
	QuickHealth.RegisterCallback(self, "HealthUpdated", function(event, GUID, newHealth)
		local unit = GUIDMap[GUID];
		local statusbar = FrameMap[unit];
		-- Note: code taken from the framexml, normaly we would setfenv it, but we can't do that without tainting it, which we rather not do :)
		if ( not statusbar ) then
			return;
		end
		if ( unit == statusbar.unit ) then
			local currValue = QuickHealth:UnitHealth(unit);
			local maxValue = UnitHealthMax(unit);

			statusbar.showPercentage = nil;

			-- Safety check to make sure we never get an empty bar.
			statusbar.forceHideText = false;
			if ( maxValue == 0 ) then
				maxValue = 1;
				statusbar.forceHideText = true;
			elseif ( maxValue == 100 ) then
				--This should be displayed as percentage.
				statusbar.showPercentage = true;
			end

			statusbar:SetMinMaxValues(0, maxValue);

			if ( not UnitIsConnected(unit) ) then
				statusbar:SetStatusBarColor(0.5, 0.5, 0.5);
				statusbar:SetValue(maxValue);
			else
				statusbar:SetStatusBarColor(0.0, 1.0, 0.0);
				statusbar:SetValue(currValue);
			end
		end
		TextStatusBar_UpdateTextString(statusbar);
	end);
end

-- Remap guids when party changed ONLY if we're not in a raid.
function handlerTable:PARTY_MEMBERS_CHANGED()
	if(GetNumRaidMembers() > 0) then return end
	RemapGUIDMap();
end

-- Remap guids when the raid roster gets updated.
handlerTable.RAID_ROSTER_UPDATE = RemapGUIDMap;
eventFrame:RegisterEvent("PARTY_MEMBERS_CHANGED");
eventFrame:RegisterEvent("RAID_ROSTER_UPDATE");
eventFrame:RegisterEvent("PLAYER_LOGIN");